@@ -4,6 +4,10 @@ import json
 import warnings
 import logging
 
+import pprint
+
+logger = logging.getLogger("modbus_server_logger")
+
 try:
     import redis
 except ImportError:
@@ -18,7 +22,7 @@ class DictDatastore:
             "input_registers": {},
             "holding_registers": {},
         }
-        logging.debug("Initialized empty DictDatastore")
+        logger.debug("Initialized empty DictDatastore")
 
     def read(self, object_reference, first_address, quantity_of_records):
         data = []
@@ -46,7 +50,7 @@ class DictDatastore:
 
 
 class RedisDatastore:
-    def __init__(self, host="localhost", port=6379, db=0, modbus_address_map={}):
+    def __init__(self, modbus_address_map={}, host="localhost", port=6379, db=0):
 
         self.host = host
         self.port = port
@@ -55,7 +59,7 @@ class RedisDatastore:
         self.r = None
         self._verify_modbus_address_map()
         self._connect()
-        logging.debug("Initialized RedisDatastore")
+        logger.debug("Initialized RedisDatastore")
 
     def _connect(self):
         self.r = redis.Redis(self.host, self.port, self.db)
@@ -83,31 +87,52 @@ class RedisDatastore:
     # TODO:
     #     self.modbus_address_map
 
+    def print_address_map(self):
+        pprint.pprint(self.modbus_address_map)
+
+    def print_all_current_values(self):
+        for object_reference, address_map in self.modbus_address_map.items():
+            print(object_reference)
+            for address, props in address_map.items():
+                print(
+                    f"\tAddress: {address} -> {props['key']} : {self.r.get(props['key'])}"
+                )
+
     def read(self, object_reference, first_address, quantity_of_records):
         data = []
         for address in range(first_address, first_address + quantity_of_records):
-            key = self.modbus_address_map[object_reference][address]["key"]
-            logging.debug(f"Getting {key} for {object_reference}:{key} from redis")
+            address_properties = self.modbus_address_map[object_reference][str(address)]
+            key = address_properties["key"]
+
             raw_value = self.r.get(key)
 
-            if object_reference in ("input_registers", "holding_registers"):
-                encoding = self.modbus_address_map[object_reference][address]["encoding"]
+            if raw_value is None:
+                logger.warning(
+                    f"Key {key} for {object_reference}:{address} not found in redis"
+                )
+                raise KeyError(f"Key {key} could not be found in redis datastore")
 
-                if encoding in ("h", "H", "i"):  # ints
-                    cast = int
+            unicode_value = raw_value.decode()
+            logger.debug(
+                f"Getting {key} for {object_reference}:{address} from redis -> {unicode_value}"
+            )
+
+            if object_reference in ("coils", "discrete_inputs"):
+                # Cast value from string to bool:
+                value = bool(distutils.util.strtobool(unicode_value))
+
+            elif object_reference in ("input_registers", "holding_registers"):
+                encoding = address_properties["encoding"]
+
+                if encoding in ("h", "H", "i", "I"):  # ints
+                    value = struct.pack(f"!{encoding}", int(unicode_value))
                 if encoding in ("e", "f", "d"):  # floats
-                    cast = float
-                value = struct.pack(f"!{encoding}", cast(raw_value))
+                    value = struct.pack(f"!{encoding}", float(unicode_value))
 
                 if encoding in ("i", "I", "f", "d"):
                     # >2 bytes: Find out which part is requested:
-                    part = self.modbus_address_map[object_reference][address]["part"]
-                    value_bytes = struct.pack(f"!{encoding}", value)
-                    value = value_bytes[part * 2 : part * 2 + 2]
-
-            elif object_reference in ("coils", "discrete_inputs"):
-                # Cast value from string to bool:
-                value = bool(distutils.util.strtobool(raw_value.decode()))
+                    part = address_properties["part"]
+                    value = value[(part - 1) * 2 : part * 2]
 
             data.append(value)
 
@@ -117,10 +142,10 @@ class RedisDatastore:
         if type(value) == bool:
             value = str(value)
         try:
-            key = self.modbus_address_map[object_reference][address]["key"]
+            key = self.modbus_address_map[object_reference][str(address)]["key"]
         except KeyError:
             key = f"{object_reference}:{address}"
-            self.modbus_address_map[object_reference][address] = {
+            self.modbus_address_map[object_reference][str(address)] = {
                 "key": key,
                 "encoding": encoding,
             }
