@@ -8,9 +8,14 @@ import logging
 # Internal imports:
 from . import modbus_datastore
 
-
 socketserver.TCPServer.allow_reuse_address = True
-logging.basicConfig(format="%(levelname)s: %(message)s", level=logging.DEBUG)
+
+logger = logging.getLogger("modbus_server_logger")
+formatter = logging.Formatter("%(levelname)s: %(message)s")
+streamhandler = logging.StreamHandler()
+streamhandler.setLevel(logging.WARNING)
+streamhandler.setFormatter(formatter)
+logger.addHandler(streamhandler)
 
 
 def pack_bools_to_bytes(bool_list):
@@ -55,7 +60,13 @@ class TCPHandler(socketserver.BaseRequestHandler):
         # Length:           (2 Bytes)   Length of the remaining frame in bytes (Total Length - 6)
         # Unit ID:          (1 Byte)    "Slave ID", inner identifier to route to different units (typically 0)
         # Function Code:    (1 Byte)    1,2,3,4,5,6,15,16,43: Read/Write input/register etc.
-        transaction_id, protocol, length, unit_id, function_code = struct.unpack("!HHHBB", self.data[:8])
+        try:
+            transaction_id, protocol, length, unit_id, function_code = struct.unpack(
+                "!HHHBB", self.data[:8]
+            )
+        except struct.error:
+            logger.error(f"Received incompatible bytes {self.data}")
+            return
 
         # Pack header items into a tuple which can more easily be passed around:
         header_items = (transaction_id, protocol, length, unit_id, function_code)
@@ -99,17 +110,19 @@ class TCPHandler(socketserver.BaseRequestHandler):
 
         try:
             # self.server is a reference to the socketserver.ThreadingTCPServer instance defined below:
-            data = self.server.datastore.read(object_reference, first_address, number_of_registers)
+            data = self.server.datastore.read(
+                object_reference, first_address, number_of_registers
+            )
         except KeyError:
             # Address not in datastore -> Respond with exception 02 - Illegal Data Address:
             response = build_error_response(header_items, exception_code=2)
-            logging.warning(
+            logger.warning(
                 f"Request from {self.client_address[0]} for {object_reference}:{first_address} -> Modbus Error 2"
             )
             self.request.sendall(response)
             return
         except:
-            logging.error(
+            logger.error(
                 f"Request from {self.client_address[0]} for {object_reference}:{first_address} -> Modbus Error 4"
             )
             raise
@@ -144,7 +157,7 @@ class TCPHandler(socketserver.BaseRequestHandler):
         # Pack response items into binary format and append data_bytes:
         response = struct.pack(f"!HHHBBB", *response_items) + data_bytes
 
-        logging.debug(
+        logger.debug(
             f"Request from {self.client_address[0]} for {object_reference}:{first_address}+{number_of_registers} -> Response {data_bytes}"
         )
 
@@ -152,7 +165,9 @@ class TCPHandler(socketserver.BaseRequestHandler):
 
 
 class Server:
-    def __init__(self, host="localhost", port=502, daemon=False, datastore=None):
+    def __init__(
+        self, host="localhost", port=502, daemon=False, datastore=None, autostart=False
+    ):
         self.host = host
         self.port = port
         if datastore is None:
@@ -161,9 +176,13 @@ class Server:
             self.datastore = datastore
         self.daemon = daemon
         self.tcp_server = None
+        if autostart:
+            self.start()
 
     def _server_thread(self):
-        self.tcp_server = socketserver.ThreadingTCPServer((self.host, self.port), TCPHandler)
+        self.tcp_server = socketserver.ThreadingTCPServer(
+            (self.host, self.port), TCPHandler
+        )
         self.tcp_server.allow_reuse_address = True
         self.tcp_server.datastore = self.datastore
         self.tcp_server.serve_forever()
@@ -173,14 +192,14 @@ class Server:
         # daemon == True -> Shutdown the thread when the main program exits
         self.server_thread.daemon = self.daemon
         self.server_thread.start()
-        logging.info("Modbus Server started")
+        logger.info("Modbus Server started")
 
     def stop(self):
         if self.tcp_server:
             self.tcp_server.shutdown()
             self.tcp_server.server_close()
             self.tcp_server = None
-        logging.info("Modbus Server stopped")
+        logger.info("Modbus Server stopped")
 
     ## Convenience Functions for direct access to object reference (single + multiple):
     ## ================================================================================
@@ -231,7 +250,9 @@ class Server:
         # Verify if value is boolean for coils and discrete inputs:
         # This is not done with bool(), because bool('0') == True might be confusing
         if object_reference in ("coils", "discrete_inputs") and not type(value) == bool:
-            raise TypeError(f"'value' for {object_reference} must be True or False, is {type(value)}")
+            raise TypeError(
+                f"'value' for {object_reference} must be True or False, is {type(value)}"
+            )
 
         # Verify if value can be converted to float for input_registers and holding_registers:
         # This works for float, int, and string with valid number inside
@@ -239,6 +260,8 @@ class Server:
 
             # Verify encoding:
             if encoding not in ("h", "H", "e"):
-                raise ValueError(f'encoding must be "h"(short), "H"(unsigned short), or "e"(float16), not {encoding}')
+                raise ValueError(
+                    f'encoding must be "h"(short), "H"(unsigned short), or "e"(float16), not {encoding}'
+                )
 
         self.datastore.write(object_reference, address, value, encoding)
